import gleam/dynamic/decode.{type Decoder}
import gleam/option.{type Option}
import gleam/time/timestamp.{type Timestamp}
import spacetraders_models/account_id.{type AccountId}
import spacetraders_models/internal/time

pub type Account {
  Account(
    id: AccountId,
    email: Option(String),
    token: Option(String),
    created_at: Timestamp,
  )
}

pub fn decoder() -> Decoder(Account) {
  use id <- decode.field("id", account_id.decoder())
  use email <- decode.optional_field(
    "email",
    option.None,
    decode.optional(decode.string),
  )
  use token <- decode.optional_field(
    "token",
    option.None,
    decode.optional(decode.string),
  )
  use created_at <- decode.field("createdAt", time.rfc3339_timestamp_decoder())
  decode.success(Account(id:, email:, token:, created_at:))
}
