import gleam/dynamic/decode.{type Decoder}
import gleam/result
import spacetraders_models/internal/jwt
import spacetraders_models/token_parse_error.{
  type TokenParseError, IncorrectType, InvalidToken,
}

pub opaque type AgentToken {
  AgentToken(String)
}

pub fn parse(value: String) -> Result(AgentToken, TokenParseError) {
  use jwt <- result.try(jwt.parse(value) |> result.replace_error(InvalidToken))
  case jwt.payload.subject {
    "agent-token" -> Ok(AgentToken(value))
    _ -> Error(IncorrectType)
  }
}

pub fn decoder() -> Decoder(AgentToken) {
  use value <- decode.then(decode.string)
  case parse(value) {
    Ok(token) -> decode.success(token)
    Error(_) -> decode.failure(AgentToken("invalid"), "AgentToken")
  }
}

pub fn to_string(agent_token: AgentToken) -> String {
  let AgentToken(value) = agent_token
  value
}
