import gleam/dynamic/decode.{type Decoder}
import gleam/option.{type Option}
import gleam/time/timestamp.{type Timestamp}
import spacetraders_models/internal/time
import spacetraders_models/ship_symbol.{type ShipSymbol}

pub type Cooldown {
  Cooldown(
    ship_symbol: ShipSymbol,
    total_seconds: Int,
    remaining_seconds: Int,
    expiration: Option(Timestamp),
  )
}

pub fn decoder() -> Decoder(Cooldown) {
  use ship_symbol <- decode.field("shipSymbol", ship_symbol.decoder())
  use total_seconds <- decode.field("totalSeconds", decode.int)
  use remaining_seconds <- decode.field("remainingSeconds", decode.int)
  use expiration <- decode.optional_field(
    "expiration",
    option.None,
    decode.optional(time.rfc3339_timestamp_decoder()),
  )
  decode.success(Cooldown(
    ship_symbol:,
    total_seconds:,
    remaining_seconds:,
    expiration:,
  ))
}
